# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

fragment_sparse_rows <- function(i, p, limits) {
    .Call('_beachmat_fragment_sparse_rows', PACKAGE = 'beachmat', i, p, limits)
}

sparse_subset_index <- function(starts, newp) {
    .Call('_beachmat_sparse_subset_index', PACKAGE = 'beachmat', starts, newp)
}

